// - Codici per i totalizzatori:
//   Questi vengono letti in G1 sulla testata:
//   SPTRAS       spese di trasporto
//   SPINC        spese incasso
//   SPIMB        spese imballaggio
//   SPBOL        spese bolli
//   Questi vengono ricalcolati in stampa:
//   TIMPONIBILI  totale imponibile (calcolato in stampa dalla tabella riepilogativa)
//   TIMPOSTE     totale imposta    (calcolato in stampa dalla tabella riepilogativa)
//   TESENTI      totale esenti     (calcolato in stampa dalla tabella riepilogativa)
//   TIMPNETTI    totale importi netti (stesso codice per il totalizzatore della riga)
//   TOTDOC       totale documento (fattura in questo caso)
//
//   Mappa dei selettori di filtro (parametro 2 di _RIEPILOGOIVA) per la stampa della tabella
//   riepilogativa e dei totali esenti/imponibili/non soggetti/non imponibili:
//   1 = regime normale
//   2 = ventilare (non usato)
//   4 = esenti
//   8 = non imponibili
//  16 = non soggetti
//   pertanto per avere i non imponibili in totale il selettore sara' 1,
//   per avere gli esenti il selettore sara' 4; per avere i non soggetti + esenti
//   il selettore sara' 20  e cosi' via.
//   Modificatori di pictures:
//   i parametri IMP_DECIMALS, QTA_DECIMALS, e PRI_DECIMALS settano i decimali da stampare/arrotondare
//   per i seguenti: importi in genere, quantita', e prezzi. Il primo parametro intero e' per gli importi in lire
//   il secondo per gli importi in valuta. Naturalmente per le quantita' non ha senso fare una cosa del genere, ma per completezza
//   e' stata implementata.
//   PRI_DECIMALS corrisponde al gruppo 29
//   QTA_DECIMALS corrisponde al gruppo 30
//   IMP_DECIMALS corrisponde al gruppo 31
//   pertanto tutti i campi che apparterranno al gruppo 29,30 o 31 avranno gli importi stampati al numero di decimali impostati a
//   seconda se il documento e' in lire o valuta. Se il numero di decimali e' 0 la picture rimarra' invariata.
//   Altri gruppi definiti dall'utente saranno cosi' sintatticamente impostati:
//   NEW_GROUP <n> <lit_dec> <val_dec>
//   Dove <n> e' il numero del gruppo
//        <lit_dec> e' il numero di decimali per i documenti in lire
//        <val_dec> e' il numero di decimali per i documenti in valuta

//   Istruzione EXCLUDE nella sezione GENERAL (facoltativo):
//   EXCLUDE esclude dalla stampa le righe che soddisfano la condizione specificata.
//   Si indicano in questo modo: "<tipo riga>,<cod. art>". Naturalmente si possono indicare tutti gli EXLCUDE che si vogliono.
//   Es: EXCLUDE "V01,ART1" escludera' tutte le righe di tipo V01 con codice articolo ART1
//       EXCLUDE ",ART1" escludera' tutte le righe che hanno codice articolo ART1
//       EXCLUDE "V01," escludera' tutte le righe che hanno tipo riga V01
//       EXCLUDE "," non escludera' nulla.


USE 34
JOIN 33 INTO CODNUM==CODNUM ANNO==ANNO PROVV==PROVV NDOC==NDOC
JOIN 16 TO 33 INTO TIPOCF==TIPOCF CODCF==CODCF CODIND==CODINDSP
JOIN 47 INTO CODART==CODART
JOIN %CPG TO 33 ALIAS 201 INTO CODTAB==CODPAG
JOIN %AGE TO 33 ALIAS 202 INTO CODTAB==CODAG
JOIN %POR TO 33 ALIAS 203 INTO CODTAB==CODPORTO
JOIN %BAN TO 33 ALIAS 204 INTO CODTAB==CODABIA+CODCABA
JOIN %VET TO 33 ALIAS 205 INTO CODTAB==CODVETT1
JOIN %ASP TO 33 ALIAS 206 INTO CODTAB==ASPBENI1
JOIN %CTR TO 33 ALIAS 207 INTO CODTAB==CAUSTRASP

END
DESCRIPTION
BEGIN
  34->* "Righe documenti di vendita"
  33->* "Documenti di vendita"
  16->* "Indirizzi di spedizione (destinazione)"
  47->* "Anagrafica magazzino"
  201@->* "Tabella condizioni di pagamento"
  202@->* "Tabella agenti"
  203@->* "Tabella porto"
  204@->* "Tabella banche"
  205@->* "Tabella vettori"
  206@->* "Tabella aspetto beni"
END

GENERAL
BEGIN
  OFFSET 0 0
  FONT "Courier New"
  SIZE 12
  MODULE "BUFFETTI_BOLLA_01"
  PRI_DECIMALS 0 3
  QTA_DECIMALS 3 3
  IMP_DECIMALS 0 3
  EXCLUDE ","
END


SECTION GRAPHICS ODD 

FIGURA 99 80 65
BEGIN
  PROMPT 1 1 "bolla.bmp"
END

END

SECTION HEADER ODD 29

STRING 1 36 1
BEGIN
 KEY "nome ditta"
 PROMPT 5 5 ""
 MESSAGE _DITTA,!RAGSOC
END

STRING 2 36 1
BEGIN
 KEY "indirizzo ditta"
 PROMPT 5 6 ""
 MESSAGE _DITTA,!IND
END

STRING 3 5 1
BEGIN
 KEY "cap ditta"
 PROMPT 5 7 ""
 MESSAGE _DITTA,!CAP
END

STRING 4 30 1
BEGIN
 KEY "comune ditta"
 PROMPT 12 7 ""
 MESSAGE _DITTA,!COM
END

STRING 5 5 1
BEGIN
 KEY "scritta P.IVA"
 PROMPT 5 8 "P.IVA"
END

STRING 6 16 1
BEGIN
 KEY "partita iva ditta"
 PROMPT 11 8 ""
 MESSAGE _DITTA,!IVA
END

STRING 7 9 1
BEGIN
 KEY "scritta COD.FISC."
 PROMPT 5 9 "COD.FISC."
END

STRING 8 16 1
BEGIN
 KEY "codice fiscale ditta"
 PROMPT 15 9 ""
 MESSAGE _DITTA,!CF
END

STRING 14 35 2
BEGIN
 KEY "destinatario, ditta"
 PROMPT 44 14 ""
 MESSAGE _CLIENTE,!RAGSOC
END

STRING 15 35 2
BEGIN
 KEY "destinatario, indirizzo con numero civico"
 PROMPT 44 16 ""
 MESSAGE _CLIENTE,!INDNUM
END

STRING 16 5 1
BEGIN
 KEY "destinatario, cap"
 PROMPT 44 18 ""
 MESSAGE _CLIENTE,!CAP
END

STRING 17 30 2
BEGIN
 KEY "destinatario, comune"
 PROMPT 50 18 ""
 MESSAGE _CLIENTE,!COM
END

STRING 18 2 
BEGIN
 KEY "destinatario, provincia"
 PROMPT 72 19 ""
 MESSAGE _CLIENTE,!PROV
END

DATE 19 8 1
BEGIN
 KEY "data bolla"
 PROMPT 60 21 ""
 FIELD 33->DATADOC
 FORMAT "1442-"
END

NUMBER 20 7
BEGIN
 KEY "numero bolla"
 PROMPT 71 21 ""
 FIELD 33->NDOC
END

STRING 21 30 2
BEGIN
 KEY "destinazione, indirizzo"
 PROMPT 5 14 ""
 FIELD 16->INDIR
END

STRING 32 5 1 
BEGIN
 KEY "destinazione, cap"
 PROMPT 5 16 ""
 FIELD 16->CAP
END

STRING 49 5 1
BEGIN
 KEY ""
 PROMPT 1 1 ""
 FLAGS "H"
 FIELD 16->COM
END

STRING 33 21 
BEGIN
 KEY "destinazione, comune"
 PROMPT 5 18 ""
 MESSAGE _ISAMREAD,13,COM=#49,DENCOM
END

STRING 34 2 1
BEGIN
 KEY "destinazione, provincia"
 PROMPT 28 18 ""
 MESSAGE _ISAMREAD,13,COM=#49,PROVCOM
END

NUMBER 22
BEGIN
 KEY "ordine"
 PROMPT 3 21 ""
 FIELD 33->NUMDOCRIF
END

DATE 23 8 1
BEGIN
 KEY "data conferma"
 PROMPT 43 21 ""
 FIELD 33->DATADOCRIF
 FORMAT "1442-"
END

STRING 24 48 1
BEGIN
 KEY "condizioni pagamento"
 PROMPT 3 23 ""
 FIELD 201@->S0
END

STRING 25 29 1
BEGIN
 KEY "banca appoggio"
 PROMPT 53 23 ""
 FIELD 204@->S0
END

STRING 26 7 1
BEGIN
 KEY "codice cliente"
 PROMPT 8 25 ""
 FIELD 33->CODCF
END

STRING 27 11 1
BEGIN
 KEY "partita iva cliente"
 PROMPT 18 25 ""
 MESSAGE _CLIENTE,!IVA
END

STRING 28 18 1
BEGIN
 KEY "porto"
 PROMPT 41 25 ""
 FIELD 203@->S0
 FLAGS "D"
END

NUMBER 29
BEGIN
 KEY "codice agente"
 PROMPT 2 25 ""
 FIELD 33->CODAG
END

STRING 30 28 1
BEGIN
 KEY "nome agente"
 PROMPT 33 29 ""
 FIELD 202@->S0
 FLAGS "D"
END

END

SECTION BODY ODD 5

STRING 32 13 1
BEGIN
 KEY "codice articolo"
 PROMPT 5 1 ""
 FIELD 34->CODART
END

STRING 33 29 4
BEGIN
 KEY "descrizione articolo"
 PROMPT 48 1 ""
 MESSAGE _DESCRIGA
END

STRING 34 2 
BEGIN
 KEY "unit di misura"
 PROMPT 30 1 ""
 FIELD 34->UMQTA
 MESSAGE _ALIGN,#33,TOP
END

NUMBER 35 10
BEGIN
 KEY "quantita"
 PROMPT 17 1 ""
 PICTURE "######"
 GROUP 30 
 FIELD 34->QTA
 MESSAGE _ALIGN,#33,TOP
END

NUMBER 36 20
BEGIN
 KEY "quantita' lettere"
 PROMPT 72 1 ""
 FIELD 34->QTA
 PICTURE "LETTERE"
 MESSAGE _ALIGN,#33,BOTTOM
 FLAGS "D"
END

END

SECTION FOOTER ODD 15

STRING 47 5 
BEGIN
 KEY "scritta SEGUE"
 PROMPT 72 13 "SEGUE"
END

END

SECTION FOOTER LAST 15

LIST 37 1 1
BEGIN
 KEY "trasporto a cura del mittente"
 PROMPT 2 4 ""
 FIELD 33->CODSPMEZZO
 ITEM "|" MESSAGE DISABLE,6@|ENABLE,5@
 ITEM "D| "
 ITEM "M|X" MESSAGE DISABLE,6@|ENABLE,5@
 ITEM "V| "
END

LIST 38 1 1
BEGIN
 KEY "trasporto a cura del destinatario"
 PROMPT 9 4 ""
 FIELD 33->CODSPMEZZO
 ITEM "|" MESSAGE DISABLE,6@|ENABLE,5@
 ITEM "D|X" MESSAGE DISABLE,6@|ENABLE,5@
 ITEM "M| "
 ITEM "V| "
END

LIST 39 1 1
BEGIN
 KEY "trasporto a cura del vettore"
 PROMPT 17 4 ""
 FIELD 33->CODSPMEZZO
 ITEM "|" // default
 ITEM "D| "
 ITEM "M| "
 ITEM "V|X" MESSAGE DISABLE,5@|ENABLE,6@
END

STRING 40 45 1
BEGIN
 KEY "nome vettore"
 PROMPT 3 6 ""
 FIELD 205@->S0
END

STRING 41 45 1
BEGIN
 KEY "indirizzo vettore"
 PROMPT 3 8 ""
 FIELD 205@->S1
END

DATE 42 8 1
BEGIN
 KEY "data inizio trasporto/consegna"
 PROMPT 48 4 ""
 FIELD 33->DATAPART
 FORMAT "1442-"
 GROUP 5
END

STRING 43 5 1
BEGIN
 KEY "ora inizio trasporto/consegna"
 PROMPT 60 4 ""
 PICTURE "## ##"
 FIELD 33->ORAPART
 GROUP 5
END

DATE 44 8 1
BEGIN
 KEY "data ritiro"
 PROMPT 51 7 ""
 FIELD 33->DATAPART
 FORMAT "1442-"
 FLAGS "D"
 GROUP 6
END

STRING 45 5 1
BEGIN
 KEY "ora ritiro"
 PROMPT 60 7 ""
 PICTURE "## ##"
 FIELD 33->ORAPART
 FLAGS "D"
 GROUP 6
END

STRING 46 27 1
BEGIN
 KEY "aspetto beni"
 PROMPT 1 8 ""
 FIELD 206@->S0
END

STRING 48 7 1
BEGIN
 KEY "peso"
 PROMPT 45 10 ""
 FIELD 33->PNETTO
END

STRING 49 5 1
BEGIN
 KEY "colli"
 PROMPT 60 10 ""
 FIELD 33->NCOLLI
END

STRING 50 15 
BEGIN
 KEY "causale"
 PROMPT 25 4 ""
 FIELD 207@->S0
END

END

END
